import { createApp } from 'vue'
import App from './App.vue'
import store, { setupStore } from '@/store'
import '../public-path'
import { setupRouter, router, createAppRouter } from '@/router/index.js'
import registerDirectives from './directives'
import setupI18n from './locales/setupI18n'
import { registerGlobComp } from '@/components/registerGlobComp'
import axios from 'axios'
// 代码高亮工具， bpmn流程图中使用，高亮 xml、json 代码
import hljsVuePlugin from '@/highlight'
import 'highlight.js/styles/atom-one-dark-reasonable.css'
// (bpmn流程图中使用) 使用tiny-emitter代替vue2.x的eventBus($on、$off、$once)
import Emitter from 'tiny-emitter'
import './permission'
import actions from './actions'
import { TOKEN_KEY } from '@/constant/app'
import { getToken } from '@/utils/auth'
import { useLangCompute } from '@/hooks/useLangCompute'
const emitter = new Emitter()
axios.interceptors.request.use(
  config => {
    if (store.getters['app/token']) {
      const token = getToken(TOKEN_KEY)
      config.headers.Authorization = 'Bearer ' + token
    }
    return config
  },
  err => {
    return Promise.reject(err)
  }
)
let instance = null
async function render(props = {}) {
  const { container } = props
  instance = createApp(App)
  instance.config.globalProperties.emitter = emitter
  instance.config.globalProperties.setGlobalState = props.setGlobalState
  instance.config.globalProperties.onGlobalStateChange = props.onGlobalStateChange
  setupStore(instance)
  instance.use(hljsVuePlugin)
  registerDirectives(instance)
  await setupI18n(instance)
  instance.config.globalProperties.$langCompute = useLangCompute
  registerGlobComp(instance)
  createAppRouter(props)
  setupRouter(instance, props)
  instance.mount(container ? container.querySelector('#vqiankun') : '#vqiankun')
}
// qiankun微前端
if (window.__POWERED_BY_QIANKUN__) {
  window.__webpack_public_path__ = window.__INJECTED_PUBLIC_PATH_BY_QIANKUN__
// eslint-disable-next-line brace-style
}
/// ////////////////无界单例模式////////////////////////
// eslint-disable-next-line indent
/* else if (window.__POWERED_BY_WUJIE__) {
  window.__WUJIE_MOUNT = () => {
    render()
  }
  window.__WUJIE_UNMOUNT = () => {
    instance.unmount()
  }
// eslint-disable-next-line brace-style
} */
/// ///////////////////////////////////////////////////
else {
  render()
}
export async function bootstrap() {
  // console.log('[vue] vue app bootstraped')
}
export async function mount(props) {
  return new Promise((resolve, reject) => {
    actions.setActions(props)
    render(props)
    // actions.setGlobalState({ qkWebLoading: false })
    /*    props.onGlobalStateChange((state, prev) => { // state: 变更后的状态; prev 变更前的状态
      console.log('子应用变更后状态', state.token)
      if (props.name.includes('data-hid') || props.name.includes('child-hid') || props.name.includes('materialChild-hid')) { setToken(TOKEN_KEY, state.token) }
    }, true) */
    resolve(props)
  })
}
export async function unmount() {
  if (instance) {
    instance.unmount()
    instance._container.innerHTML = ''
    instance = null
  }
}
/**
 * 可选生命周期钩子，仅使用 loadMicroApp 方式加载微应用时生效
 */
export async function update(props) {
  console.log('update props', props)
  router.push(props.path)
}
// 换成cdn图标
const element = document.createElement('script')
element.src = 'http://drd.haier.net/obs/hidpicons/iconfont.js'
document.body.appendChild(element)
