import { markRaw } from 'vue'
import { createRouter, createWebHashHistory, createMemoryHistory } from 'vue-router'
import { importAll, /* setDocumentTitle */ } from '@/utils'
import Login from '@/views/login/index.vue'
import UserLogin from '@/views/userlogin/index.vue'
import { permissionJudge } from '@/permission'
export const asyncRoutes = generateAsyncRoutes()

export const defaultRoutes = [
  {
    path: '/',
    redirect: '/lpm',
    hidden: true,
  },
  {
    path: '/userlogin',
    name: 'userlogin',
    hidden: true,
    component: markRaw(UserLogin),
  },
  {
    path: '/login',
    name: 'login',
    hidden: true,
    component: markRaw(Login),
  },
  {
    path: '/401',
    name: '401',
    hidden: true,
    component: markRaw(import('@/views/error/401.vue')),
  },
  ...asyncRoutes,
]

export const errorRoute = {
  path: '/:notFoundPath(.*)*',
  hidden: true,
  name: 'notFound',
  component: () => markRaw(import(/* webpackChunkName: 'error' */ '@/views/error/404.vue')),
}
let routeMode = null
let router = null

export function createAppRouter(props) {
  if (props && props.mode === 'abstract') {
    routeMode = createMemoryHistory()
  } else {
    routeMode = createWebHashHistory()
  }
  router = createRouter({
    history: routeMode,
    scrollBehavior: () => ({ top: 0, left: 0 }),
    routes: [...defaultRoutes, errorRoute]
  })
  permissionJudge(router)
}
export { router }

function generateAsyncRoutes() {
  const modulesFiles = require.context('./modules', false, /\.js$/)
  const modules = importAll(modulesFiles)
  let asyncRoutes = []
  for (const key in modules) {
    asyncRoutes = asyncRoutes.concat(modules[key])
  }
  return asyncRoutes
  // return []
}

export function setupRouter(app) {
  app.use(router)
}
export default router
