// import { DEFAULT_INDEX } from '@/constant/app'
import _ from 'lodash-es'
import { ElMessage } from 'element-plus'
import store from '@/store'
/**
  * 导入指定上下文中所有模块的默认导出，导出结果以文件名为key，默认导出为value
  */
export function importAll(modulesFiles, bAllExport = false) {
  const modules = modulesFiles.keys().reduce((modules, modulePath) => {
    if (modulePath.includes('index.js')) {
      return modules
    }
    const moduleName = modulePath.replace(/^\.\/(.*)\.\w+$/, '$1')
    const value = modulesFiles(modulePath)
    modules[moduleName] = bAllExport ? {
      ...value,
    } : value.default
    return modules
  }, {})
  return modules
}

export function importLang(modulesFiles, language) {
  const modules = modulesFiles.keys().reduce((modules, modulePath) => {
    let moduleName = './' + language + '/' + modulePath.replace(/^\.\/(.*)\.\w+$/, '$1')
    if (modulePath.includes('index.js')) {
      moduleName = moduleName.replace('/index', '')
    }
    const value = modulesFiles(modulePath)
    modules[moduleName] = value.default
    return modules
  }, {})
  return modules
}

/**
 * 设置文档标题，路由名 - 系统名
 * @param {string} title 文档标题
 */
export function setDocumentTitle(title) {
  document.title = title || process.env.VUE_APP_TITLE
}

/** 数组扁平化处理
 * @param {Array} arr 需要扁平化的对象数组，如 [{},{ children: [{}] }]
 */
export function flatArray(arr) {
  const flatArr = []
  arr.forEach(item => {
    flatArr.push({ ...item })
    if (item.children && item.children.length !== 0) {
      flatArr.push(
        ...flatArray(item.children)
      )
    }
  })
  return flatArr
}

export function getTableHeight(differH) {
  const h = (document.documentElement.clientHeight - differH) + ''
  return h
}

export function parseTime(time, cFormat) {
  if (arguments.length === 0 || !time) {
    return null
  }
  const format = cFormat || '{y}-{m}-{d} {h}:{i}:{s}'
  let date
  if (typeof time === 'object') {
    date = time
  } else {
    if ((typeof time === 'string')) {
      if ((/^[0-9]+$/.test(time))) {
        // support "1548221490638"
        time = parseInt(time)
      } else {
        // support safari
        // https://stackoverflow.com/questions/4310953/invalid-date-in-safari
        time = time.replace(new RegExp(/-/gm), '/')
      }
    }

    if ((typeof time === 'number') && (time.toString().length === 10)) {
      time = time * 1000
    }
    date = new Date(time)
  }
  const formatObj = {
    y: date.getFullYear(),
    m: date.getMonth() + 1,
    d: date.getDate(),
    h: date.getHours(),
    i: date.getMinutes(),
    s: date.getSeconds(),
    a: date.getDay()
  }
  const timeStr = format.replace(/{([ymdhisa])+}/g, (result, key) => {
    const value = formatObj[key]
    // Note: getDay() returns 0 on Sunday
    if (key === 'a') { return ['日', '一', '二', '三', '四', '五', '六'][value] }
    return value.toString().padStart(2, '0')
  })
  return timeStr
}

export function compareDateUtil(t1, t2) {
  var date = new Date()
  var a = t1.split(':')
  var b = t2.split(':')
  return date.setHours(a[0], a[1]) >= date.setHours(b[0], b[1])
}

export function isHourMinutes(time) {
  const reg = /([0-1][0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9])/
  return reg.test(time)
}

export function getHourDiffUtil(s1, s2) {
  s1 = new Date(('1970-1-1 ' + s1).replace(/-/g, '/'))
  s2 = new Date(('1970-1-1 ' + s2).replace(/-/g, '/'))
  var ms = s2.getTime() - s1.getTime()
  if (ms < 0) return false
  // console.log(typeof (ms / 1000 / 60 / 60))
  return ms / 1000 / 60 / 60
}

export function downloadByUrl(url) {
  var a = document.createElement('a')
  document.body.appendChild(a)
  a.target = 'tempiframe'
  a.href = url
  a.setAttribute('download', url)
  a.click()
  window.URL.revokeObjectURL(url)
  document.body.removeChild(a)
}

export function genNonDuplicateID(randomLength) {
  return Number(Math.random().toString().substr(3, randomLength) + Date.now()).toString(36)
}

export const redirectToSingleLogin = (data, exit) => {
  if (!data) {
    ElMessage({
      showClose: true,
      message: 'Network Error',
      type: 'error',
      duration: 5 * 1000
    })
    return
  }
  const { clientId, ssoUrl } = data
  // eslint-disable-next-line no-useless-escape
  const u = window.location.href.replace(/\??\&?code=.*|\??\&?language=.*|\??\&?state=.*/g, '')
  // const index = u.indexOf('/#/')
  // let returnUrl
  // if (index !== -1) {
  //   const temp = u.substring(index, u.length)
  //   returnUrl = u.replace(temp, '/#' + DEFAULT_INDEX)
  // } else {
  //   returnUrl = u
  // }
  const returnUrl = u
  let jumpto
  if (exit) {
    // jumpto = ssoUrl + '?&client_id=' + clientId + '&returnUrl=' + encodeURIComponent(returnUrl) + '&exitUrl=' + encodeURIComponent(returnUrl) + '#/exit'
    jumpto = `${ssoUrl}?client_id=${clientId}&redirect_uri=${encodeURIComponent(returnUrl)}#exit`
  } else {
    jumpto = `${ssoUrl}?client_id=${clientId}&redirect_uri=${encodeURIComponent(returnUrl)}`
  }
  window.location.replace(jumpto)
}

export function hasDuplicates(a) {
  return _.uniq(a).length !== a.length
}

export const formatCurrentToken = (t) => {
  return t.replace('Bearer ', '') || ''
}

export const mouseFollowClass = (ev) => {
  var l = ev.target.getBoundingClientRect().left
  var t = ev.target.getBoundingClientRect().top
  var w = ev.target.getBoundingClientRect().width
  var h = ev.target.getBoundingClientRect().height
  var x = (ev.clientX - l - w / 2) * (w > h ? (h / w) : 1)
  var y = (ev.clientY - t - h / 2) * (h > w ? (w / h) : 1)
  var deg = Math.atan2(y, x) / (Math.PI / 180)
  var d = (Math.round((deg + 180) / 90) + 3) % 4
  let transitionClass = ''
  switch (d) {
    case 0:
      transitionClass = 'fade-slide-top'
      break
    case 1:
      transitionClass = 'fade-slide-right'
      break
    case 2:
      transitionClass = 'fade-slide-bottom'
      break
    default:
      transitionClass = 'fade-slide-left'
      break
  }
  return transitionClass
}

export const downloadFile = (data, filename) => {
  const link = document.createElement('a')
  const blob = new Blob([data], { type: 'application/vnd.ms-excel' })
  link.style.display = 'none'
  link.href = URL.createObjectURL(blob)
  link.setAttribute('download', `${filename}.xlsx`)
  document.body.appendChild(link)
  link.click()
  document.body.removeChild(link)
}

export const downloadFileKnowledge = (res, filename, type, suffix) => {
  let blob = null
  if (type) {
    blob = new Blob([res.data], { type })
  } else {
    blob = new Blob([res.data])
  }
  const url = window.URL.createObjectURL(blob)
  const link = document.createElement('a')
  link.style.display = 'none'
  if (suffix) { // 有后缀是给批量下载.zip格式的文件准备的
    link.download = filename + suffix
  } else {
    const contentDisposition = decodeURI(res.headers['content-disposition']) || ''
    let currentFileName = ''
    if (contentDisposition) {
      const patt = new RegExp('filename=([^;]+\\.[^\\.;]+);*')
      const result = patt.exec(contentDisposition)
      if (result instanceof Array) {
        currentFileName = result[1] || ''
      } else {
        currentFileName = ''
      }
    } else {
      currentFileName = ''
    }
    link.download = currentFileName || filename
  }
  link.href = url
  document.body.appendChild(link)
  link.click()
  document.body.removeChild(link)
}

// 根据按钮权限集合判断按钮是否显示
export const hasBtnPermission = (val) => {
  const buttons = store.getters['app/buttons']
  if (!buttons) {
    return false
  }
  const btn = buttons[val] || null
  if (!btn) {
    return false
  } else {
    return true
  }
}
export function getSessionUserInfo() {
  return JSON.parse(window.sessionStorage.getItem('userInfo'))?.content
}
// 获取当前用户信息
export function getUserInfos() {
  return getSessionUserInfo() ? getSessionUserInfo() : store.getters['app/userInfo'] ?? {}
  // return JSON.parse(window.sessionStorage.getItem('userInfo') as string)?.content ?? {};
}
